package com.iden.bms.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.iden.bms.service.FaceImageService;
import com.iden.common.annotation.Permission;
import com.iden.common.exception.BDException;
import com.iden.common.util.PageResult;
import com.iden.common.util.Result;
import com.iden.common.util.WebPageUtils;
import com.iden.common.vo.PageReqVO;
import com.iden.common.vo.UserLoginedConvertVO;
import com.iden.common.vo.FaceImageVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "FaceImageController", tags = { "人脸抓拍库管理" })
@Slf4j
@RequestMapping("/bms/faceImage")
@Permission
public class FaceImageController {

    @Autowired
    private FaceImageService faceImageService;



    @GetMapping("/listFaceImage")
    @ApiOperation(value = "人脸抓拍库列表分页 ")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "type", value = "类型，1：陌生人员 ，2重点人员，3小区人员"),
            @ApiImplicitParam(paramType = "query", name = "communityId", value = "小区ID"),
            @ApiImplicitParam(paramType = "query", name = "photographPlace", value = "抓拍地点"),
            @ApiImplicitParam(paramType = "query", name = "beginDate", value = "开始时间(yyyyMMdd)"),
            @ApiImplicitParam(paramType = "query", name = "endDate", value = "结束时间(yyyyMMdd)")

    })
    public PageResult<List<FaceImageVO>> listFaceImage(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                       @RequestParam(value = "type", required = true) String type,
                                                       @RequestParam(value = "communityId", required = false) Long communityId,
                                                       @RequestParam(value = "photographPlace", required = false) String photographPlace,
                                                       @RequestParam(value = "beginDate", required = false) String beginDate,
                                                       @RequestParam(value = "endDate", required = false) String endDate,
                                                       PageReqVO pageReqVo){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<FaceImageVO> pageResponse = this.faceImageService.listFaceImage(type,communityId,photographPlace,beginDate,endDate,loginUser ,pageReqVo);
            return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
        }catch (BDException e) {
            log.error("人脸抓拍库列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("人脸抓拍库管理: 人脸抓拍库列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }

    @GetMapping("/getFaceImageInfo/{id}")
    @ApiOperation(value = "人脸抓拍库")
    public Result<FaceImageVO> getFaceImageInfo(HttpServletRequest request, @RequestHeader("token") String token, @PathVariable("id") Long id){
        FaceImageVO vo = this.faceImageService.getFaceImageInfo(id);
        return  Result.success("查询成功！",vo);
    }
}
