package com.iden.bms.controller;


import com.iden.bms.service.HomePageService;
import com.iden.common.annotation.Permission;

import com.iden.common.util.Result;
import com.iden.common.util.WebPageUtils;
import com.iden.common.vo.HomPageLastWeekFaceImageStatVO;
import com.iden.common.vo.HomPageStaVO;
import com.iden.common.vo.HomePageWarningStaVO;
import com.iden.common.vo.UserLoginedConvertVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "HomePageController", tags = { "首页管理" })
@Slf4j
@RequestMapping("/bms/homePage")
@Permission
public class HomePageController {

    @Autowired
    private HomePageService homePageService;

    @GetMapping("/queryHomPageTotal")
    @ApiOperation(value = "查询首页汇总值")
    public Result<HomPageStaVO> queryHomPageTotal(HttpServletRequest request, @RequestHeader(value = "token") String token){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            HomPageStaVO datas = this.homePageService.queryHomPageTotal(loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("首页管理: 查询首页汇总值出现异常",e);
            return Result.error( "获取数据失败");
        }
    }

    @GetMapping("/queryWarningPersonRateStat")
    @ApiOperation(value = "查询预警人员占比统计")
    public Result<List<HomePageWarningStaVO>> queryWarningPersonRateStat(HttpServletRequest request, @RequestHeader(value = "token") String token){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            List<HomePageWarningStaVO> datas = this.homePageService.queryWarningPersonRateStat(loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("首页管理: 查询预警人员占比统计出现异常",e);
            return Result.error( "获取数据失败");
        }
    }


    @GetMapping("/queryWarningEventRateStat")
    @ApiOperation(value = "查询预警事件占比统计")
    public Result<List<HomePageWarningStaVO>> queryWarningEventRateStat(HttpServletRequest request, @RequestHeader(value = "token") String token){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            List<HomePageWarningStaVO> datas = this.homePageService.queryWarningEventRateStat(loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("首页管理: 查询预警事件占比统计出现异常",e);
            return Result.error( "获取数据失败");
        }
    }


    @GetMapping("/listLastWeekFaceImageSta")
    @ApiOperation(value = "查询上周人脸抓拍统计")
    public Result<List<HomPageLastWeekFaceImageStatVO>> listLastWeekFaceImageSta(HttpServletRequest request, @RequestHeader(value = "token") String token){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            List<HomPageLastWeekFaceImageStatVO> datas = this.homePageService.listLastWeekFaceImageSta(loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("首页管理: 查询上周统计出现异常",e);
            return Result.error( "获取列表失败");
        }
    }

    @GetMapping("/listLastWeekStrangerFaceImageSta")
    @ApiOperation(value = "查询上周陌生人脸抓拍统计")
    public Result<List<HomPageLastWeekFaceImageStatVO>> listLastWeekStrangerFaceImageSta(HttpServletRequest request, @RequestHeader(value = "token") String token){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            List<HomPageLastWeekFaceImageStatVO> datas = this.homePageService.listLastWeekStrangerFaceImageSta(loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("首页管理: 查询上周陌生人脸抓拍统计出现异常",e);
            return Result.error( "获取列表失败");
        }
    }

    @GetMapping("/listLastWeekKeynoteFaceImageSta")
    @ApiOperation(value = "查询上周重点人脸抓拍统计")
    public Result<List<HomPageLastWeekFaceImageStatVO>> listLastWeekKeynoteFaceImageSta(HttpServletRequest request, @RequestHeader(value = "token") String token){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            List<HomPageLastWeekFaceImageStatVO> datas = this.homePageService.listLastWeekKeynoteFaceImageSta(loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("首页管理: 查询上周重点人脸抓拍统计出现异常",e);
            return Result.error( "获取列表失败");
        }
    }

    @GetMapping("/listLastWeekEventComingFaceImageSta")
    @ApiOperation(value = "查询上周昼伏夜出人脸抓拍统计")
    public Result<List<HomPageLastWeekFaceImageStatVO>> listLastWeekEventComingFaceImageSta(HttpServletRequest request, @RequestHeader(value = "token") String token){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            List<HomPageLastWeekFaceImageStatVO> datas = this.homePageService.listLastWeekEventComingFaceImageSta(loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("首页管理: 查询上周昼伏夜出人脸抓拍统计出现异常",e);
            return Result.error( "获取列表失败");
        }
    }

}
