package com.iden.bms.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.iden.bms.params.ModifyPasswordParams;
import com.iden.bms.service.SysUserService;
import com.iden.common.annotation.Permission;
import com.iden.common.exception.BDException;
import com.iden.common.logaspect.LogAnnotation;
import com.iden.common.logaspect.OperateType;
import com.iden.common.util.PageResult;
import com.iden.common.util.Result;
import com.iden.common.util.WebPageUtils;
import com.iden.common.vo.PageReqVO;
import com.iden.common.vo.UserLoginedConvertVO;
import com.iden.common.vo.SysUserVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "SysUserController", tags = { "系统用户管理" })
@Slf4j
@RequestMapping("/bms/sysuser")
@Permission
public class SysUserController {

    @Autowired
    private SysUserService sysUserService;


    @GetMapping("/listSysUser")
    @ApiOperation(value = "系统用户列表分页 ")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "nickname", value = "用户姓名"),
            @ApiImplicitParam(paramType = "query", name = "username", value = "用户名")
    })
    public PageResult<List<SysUserVO>> listSysUser(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                   @RequestParam(value = "nickname", required = false) String nickname,
                                                   @RequestParam(value = "username", required = false) String username,
                                                   PageReqVO pageReqVo){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<SysUserVO> pageResponse = this.sysUserService.listSysUser(nickname,username,loginUser ,pageReqVo);
            return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
        }catch (BDException e) {
            log.error("系统用户列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("系统用户管理: 系统用户列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }

    @PostMapping("/addSysUser")
    @ApiOperation(value = "新增系统用户")
    @LogAnnotation(
            type = OperateType.ADD,
            moduleName = "新增系统用户",
            description = "新增系统用户"
    )
    public Result<Object> addSysUser(HttpServletRequest request,@RequestHeader("token") String token,
                                   @RequestBody SysUserVO vo){
        try {

            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            String md5Password = DigestUtils.md5Hex("123456");
            vo.setPassword(md5Password);
            int flag = this.sysUserService.createSysUser(vo,loginUser);
            if (flag == 1) {
                return  Result.error("用户名已存在！");
            } else if (flag == 0) {
                return  Result.success("新增成功！");
            } else {
                return  Result.error("新增失败！");
            }

        }catch (BDException e) {
            log.error("新增系统用户-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("系统用户管理: 新增系统用户出现异常",e);
            return Result.error("新增失败！");
        }
    }

    @PostMapping("/updateSysUser")
    @ApiOperation(value = "修改系统用户")
    @LogAnnotation(
            type = OperateType.MODIFY,
            moduleName = "修改系统用户",
            description = "修改系统用户"
    )
    public Result<Object> updateSysUser(HttpServletRequest request,@RequestHeader("token") String token,
                                 @RequestBody SysUserVO vo){
        try {
            this.sysUserService.updateSysUser(vo);
            return  Result.success("修改成功！");
        }catch (BDException e) {
            log.error("修改系统用户-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("系统用户管理: 修改系统用户出现异常",e);
            return Result.error("修改失败！");
        }
    }

    @PostMapping("/invalidSysUser")
    @ApiOperation(value = "启用停用（只设置id和validStatus（0启用，1停用））")
    @LogAnnotation(
            type = OperateType.MODIFY,
            moduleName = "启用停用",
            description = "启用停用"
    )
    public Result<Object> invalidSysUser(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                         @RequestBody SysUserVO vo){
        try {
            sysUserService.invalidSysUser(vo);
            return Result.success("操作成功！");
        }catch (BDException e) {
            log.error("启用停用用户出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("用户管理: 启用停用用户出现异常",e);
            return Result.error("启用停用用户失败！");
        }
    }

    @GetMapping("/getSysUserInfo/{id}")
    @ApiOperation(value = "系统用户详情")
    public Result<SysUserVO> getSysUserInfo(HttpServletRequest request, @RequestHeader("token") String token, @PathVariable("id") Long id){
        SysUserVO orderInfo = this.sysUserService.getSysUserById(id);
        return  Result.success("查询成功！",orderInfo);
    }

    @PostMapping("/deleteSysUser/{id}")
    @ApiOperation(value = "删除系统用户")
    @LogAnnotation(
            type = OperateType.REMOVE,
            moduleName = "删除系统用户",
            description = "删除系统用户"
    )
    public Result<Object> deleteSysUser(HttpServletRequest request,@RequestHeader("token") String token,
                                 @PathVariable("id") Long id){
        try {
            if( this.sysUserService.deleteById(id)){
                return  Result.success("删除成功！");
            } else {
                return Result.error("系统用户被引用，删除失败！");
            }

        } catch (Exception e) {
            log.error("系统用户管理: 删除出现异常",e);
            return Result.error("删除失败！");
        }
    }


    @PostMapping("/resetPassword/{id}")
    @ApiOperation(value = "重置密码")
    @LogAnnotation(
            type = OperateType.MODIFY,
            moduleName = "重置密码",
            description = "重置密码"
    )
    public Result<Object> resetPassword(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                      @PathVariable("id") Long id){
        try {

            if (sysUserService.resetPassword(id)){
                return  Result.success("重置成功！");
            } else {
                return  Result.error("重置失败！");
            }

        }catch (BDException e) {
            log.error("重置密码-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("用户管理: 重置密码出现异常",e);
            return Result.error("重置密码失败！");
        }
    }

    @PostMapping("/modifyPassword")
    @ApiOperation(value = "修改密码")
    @LogAnnotation(
            type = OperateType.MODIFY,
            moduleName = "修改密码",
            description = "修改密码"
    )
    public Result<Object> modifyPassword(HttpServletRequest request,
                                         @RequestHeader("token") String token,
                                         @RequestBody ModifyPasswordParams parmas){
        try {
            if(StringUtils.isEmpty(parmas.getNewPassword()) || !parmas.getNewPassword().equals(parmas.getConfirmNewPassword())){
                return  Result.error("新密码和确认新密码不一致！");
            }
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);

            if (sysUserService.modifyPassword(loginUser.getId(),parmas)){
                return  Result.success("修改密码成功！");
            } else {
                return  Result.error("修改密码失败！");
            }

        }catch (BDException e) {
            log.error("修改密码-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("用户管理: 修改密码出现异常",e);
            return Result.error("修改密码失败！");
        }
    }
}
