package com.iden.bms.controller;



import com.iden.bms.service.TraceService;
import com.iden.common.annotation.Permission;
import com.iden.common.exception.BDException;

import com.iden.common.logaspect.LogAnnotation;
import com.iden.common.logaspect.OperateType;
import com.iden.common.util.Result;
import com.iden.common.util.WebPageUtils;
import com.iden.common.vo.CameraVideoVO;
import com.iden.common.vo.TraceVO;
import com.iden.common.vo.UserLoginedConvertVO;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "TraceController", tags = { "轨迹管理" })
@Slf4j
@RequestMapping("/bms/trace")
@Permission
public class TraceController {

    @Autowired
    private TraceService traceService;

    @ApiOperation(value = "图像上传")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "放在请求头中的令牌",
                    dataType = "String", paramType = "header",
                    required = true)
    })
    @CrossOrigin
    @PostMapping(value = "/uploadImage", headers="content-type=multipart/form-data",produces = "application/json;charset=UTF-8")
    @LogAnnotation(
            type = OperateType.UPLOAD,
            moduleName = "上传图像获取目标轨迹",
            description = "上传图像获取目标轨迹"
    )
    public Result<String> uploadImage(@ApiParam(value="图像",required=true) MultipartFile file,
                                      HttpServletRequest request, @RequestHeader(name = "token") String token) {
        try {
            String imageName = traceService.uploadImage(file);
            return  Result.success("上传成功！",imageName);
        } catch (BDException e) {
            log.error("图像上传出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("轨迹管理: 图像上传出现异常",e);
            return Result.error("图像上传失败！");
        }
    }


    @GetMapping("/listUploadImagesTrace")
    @ApiOperation(value = "使用上传图像获取目标轨迹列表（地图打点划线） ")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "imageNames", value = "图像名称，多个用逗号分割"),
            @ApiImplicitParam(paramType = "query", name = "beginTime", value = "开始时间(yyyyMMdd HH:mm:ss)"),
            @ApiImplicitParam(paramType = "query", name = "endTime", value = "结束时间(yyyyMMdd HH:mm:ss)")

    })
    public Result<List<TraceVO>> listUploadImagesTrace(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                       @RequestParam(value = "imageNames", required = true) String imageNames,
                                                       @RequestParam(value = "beginTime", required = false) String beginTime,
                                                       @RequestParam(value = "endTime", required = false) String endTime){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            List<TraceVO> datas = this.traceService.listUploadImagesTrace(imageNames,beginTime,endTime,loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("轨迹管理: 使用上传图像获取目标轨迹列表出现异常",e);
            return Result.error( "获取列表失败");
        }
    }


    @GetMapping("/getCameraVideoList")
    @ApiOperation(value = "获取摄像机视频列表")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "cameraId", value = "摄像机ID"),
            @ApiImplicitParam(paramType = "query", name = "beginTime", value = "开始时间(yyyyMMdd HH:mm:ss)"),
            @ApiImplicitParam(paramType = "query", name = "endTime", value = "结束时间(yyyyMMdd HH:mm:ss)")

    })
    public Result<CameraVideoVO> getCameraVideoList(HttpServletRequest request, @RequestHeader("token") String token,
                                                 @RequestParam(value = "cameraId", required = false) Long cameraId,
                                                 @RequestParam(value = "beginTime", required = false) String beginTime,
                                                 @RequestParam(value = "endTime", required = false) String endTime){
        this.traceService.getCameraVideoList(cameraId,beginTime,endTime);
        return  Result.success("查询成功！");
    }

    /**
     * 轨迹下载
     * @param
     * @return
     */
    @ApiOperation(value = "轨迹下载")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "district", value = "所属区域"),
            @ApiImplicitParam(paramType = "query", name = "subdistrict", value = "所属街道"),
            @ApiImplicitParam(paramType = "query", name = "name", value = "小区名称")
    })
    @PostMapping({"/exportToExcel"})
    @LogAnnotation(
            type = OperateType.EXPORT,
            moduleName = "轨迹下载",
            description = "轨迹下载"
    )
    public void exportToExcel( HttpServletRequest request,@RequestHeader(name = "token", required = true) String token,
                              HttpServletResponse response,
                              @RequestBody List<TraceVO> traceVOS) {
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            this.traceService.exportToExcel(traceVOS,loginUser ,response);
        } catch (BDException e) {
            log.error("导出小区列表出现异常",e);
        } catch (Exception e) {
            log.error("小区管理: 导出小区列表出现异常",e);
        }
    }
}
