package com.iden.bms.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.iden.bms.service.TraceService;
import com.iden.bms.service.WarningPersonService;
import com.iden.common.annotation.Permission;
import com.iden.common.exception.BDException;
import com.iden.common.logaspect.LogAnnotation;
import com.iden.common.logaspect.OperateType;
import com.iden.common.util.PageResult;
import com.iden.common.util.Result;
import com.iden.common.util.WebPageUtils;
import com.iden.common.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "WarningPersonController", tags = { "预警管理" })
@Slf4j
@RequestMapping("/bms/warningPersion")
@Permission
public class WarningPersonController {
    @Autowired
    private WarningPersonService warningPersonService;
    @Autowired
    private TraceService traceService;

    @GetMapping("/listWarningPersonbyCommunity")
    @ApiOperation(value = "根据小区查询预警人员列表分页 ")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "type", value = "预警类型，1 陌生人员预警 2重点人员预警 3常住人员预警 4昼伏夜出人员预警"),
            @ApiImplicitParam(paramType = "query", name = "communityId", value = "小区ID"),
            @ApiImplicitParam(paramType = "query", name = "nameOrCred", value = "名称/身份证"),
            @ApiImplicitParam(paramType = "query", name = "populationType", value = "人口类型"),
            @ApiImplicitParam(paramType = "query", name = "crowdId", value = "人群类型ID"),
            @ApiImplicitParam(paramType = "query", name = "beginDate", value = "开始时间(yyyyMMdd)"),
            @ApiImplicitParam(paramType = "query", name = "endDate", value = "结束时间(yyyyMMdd)"),
            @ApiImplicitParam(paramType = "query", name = "handleStatus", value = "处理状态，0未处理，1已处理")

    })
    public PageResult<List<WarningPersonVO>> listWarningPersonbyCommunity(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                                          @RequestParam(value = "type", required = true) String type,
                                                                          @RequestParam(value = "communityId", required = true) Long communityId,
                                                                          @RequestParam(value = "nameOrCred", required = false) String nameOrCred,
                                                                          @RequestParam(value = "populationType", required = false) String populationType,
                                                                          @RequestParam(value = "crowdId", required = false) Long crowdId,
                                                                          @RequestParam(value = "beginDate", required = false) String beginDate,
                                                                          @RequestParam(value = "endDate", required = false) String endDate,
                                                                          @RequestParam(value = "handleStatus", required = false) String handleStatus,
                                                                          PageReqVO pageReqVo){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<WarningPersonVO> pageResponse = this.warningPersonService.listWarningPersonbyCommunity(type,communityId,nameOrCred,populationType,crowdId,beginDate,endDate,handleStatus,loginUser ,pageReqVo);
            return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
        }catch (BDException e) {
            log.error("根据小区查询预警人员列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("预警管理: 根据小区查询预警人员列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }


    @GetMapping("/listTrace")
    @ApiOperation(value = "点击预警人员获取目标轨迹列表（地图打点划线） ")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "uid", value = "唯一生物学标识"),
            @ApiImplicitParam(paramType = "query", name = "beginDate", value = "开始时间(yyyyMMdd)"),
            @ApiImplicitParam(paramType = "query", name = "endDate", value = "结束时间(yyyyMMdd)")

    })
    public Result<List<TraceVO>> listTrace(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                       @RequestParam(value = "uid", required = true) String uid,
                                                       @RequestParam(value = "beginDate", required = false) String beginDate,
                                                       @RequestParam(value = "endDate", required = false) String endDate){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            List<TraceVO> datas = this.warningPersonService.listTrace(uid,beginDate,endDate,loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("预警管理: 点击预警人员获取目标轨迹列表出现异常",e);
            return Result.error( "获取列表失败");
        }
    }

    @PostMapping("/handleWarning")
    @ApiOperation(value = "处理预警（WarningPersonVO必须设置id和type）")
    @LogAnnotation(
            type = OperateType.MODIFY,
            moduleName = "处理预警",
            description = "处理预警"
    )
    public Result<Object> handleWarning( HttpServletRequest request,@RequestHeader("token") String token,
                                         @RequestBody WarningPersonVO vo){
        try {
            if(warningPersonService.handleWarning(vo)){
                return Result.success("处理预警成功！");
            } else {
                return Result.error("处理预警失败！");
            }

        }catch (BDException e) {
            log.error("处理预警出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("预警管理: 处理预警出现异常",e);
            return Result.error("处理预警失败！");
        }
    }

    @GetMapping("/listEventTimes")
    @ApiOperation(value = "预警事件时间列表")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "id", value = "该条数据主键")
    })
    public Result<List<WarningEventVO>> listEventTimes(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                           @RequestParam(value = "id", required = true) Long id){
        try {
            UserLoginedConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            List<WarningEventVO> datas = this.warningPersonService.listEventTimes(id,loginUser);
            return Result.success(datas);
        }catch (Exception e) {
            log.error("预警管理: 预警事件时间列表查询出现异常",e);
            return Result.error( "获取列表失败");
        }
    }

}
