package com.iden.bms.controller;


import com.iden.bms.service.WarningRuleService;
import com.iden.common.annotation.Permission;
import com.iden.common.exception.BDException;
import com.iden.common.logaspect.LogAnnotation;
import com.iden.common.logaspect.OperateType;
import com.iden.common.util.Result;
import com.iden.common.vo.WarningRuleVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import javax.servlet.http.HttpServletRequest;


/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "WarningRuleController", tags = { "预警规则管理" })
@Slf4j
@RequestMapping("/bms/warningRule")
@Permission
public class WarningRuleController {

    @Autowired
    private WarningRuleService warningRuleService;

    @PostMapping("/updateWarningRule")
    @ApiOperation(value = "修改预警规则")
    @LogAnnotation(
            type = OperateType.EDIT,
            moduleName = "修改预警规则",
            description = "修改预警规则"
    )
    public Result<Object> updateWarningRule(HttpServletRequest request,@RequestHeader("token") String token,
                                 @RequestBody WarningRuleVO vo){
        try {
            this.warningRuleService.updateWarningRule(vo);
            return  Result.success("修改成功！");
        }catch (BDException e) {
            log.error("修改预警规则-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("预警规则管理: 修改预警规则出现异常",e);
            return Result.error("修改失败！");
        }
    }

    @GetMapping("/getAllWarningRuleInfo")
    @ApiOperation(value = "全部预警规则详情")
    public Result<WarningRuleVO> getAllWarningRuleInfo(HttpServletRequest request, @RequestHeader("token") String token){
        WarningRuleVO orderInfo = this.warningRuleService.getAllWarningRuleInfo();
        return  Result.success("查询成功！",orderInfo);
    }

    @GetMapping("/getWarningRuleInfoByType")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "type", value = "预警类型，1 陌生人员预警 2重点人员预警 3常住人员预警 4昼伏夜出人员预警")
    })
    @ApiOperation(value = "全部预警规则详情")
    public Result<WarningRuleVO> getWarningRuleInfoByType(HttpServletRequest request, @RequestHeader("token") String token,
                                                          @RequestParam(value = "type", required = false) String type){
        WarningRuleVO orderInfo = this.warningRuleService.getWarningRuleInfoByType(type);
        return  Result.success("查询成功！",orderInfo);
    }

}
