package com.iden.bms.schedule;



import com.iden.bms.service.WarningService;
import com.iden.common.cache.RedisKeyConstant;
import com.iden.common.cache.RedisUtil;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.util.UUID;

/**
 * 把预警人员的轨迹入库
 */
@Configuration
@EnableScheduling
@EnableAsync
public class WarningSchedule {

    @Resource
    private WarningService warningService;
    @Resource
    private RedisUtil redisUtil;

    /**
     * 实时处理轨迹、重点人员预警等
     */
    @Async
    @Scheduled(cron = "20 0/5 * * * ?")
    public void handleActualTimeWarning() {
        String key = RedisKeyConstant.HANDLE_KEYNOTE_WARNING;
        String requestId = UUID.randomUUID().toString();
        boolean result = redisUtil.tryLock(key,requestId,5 * 60);
        try {
            if (result) {
                warningService.handleActualTimeWarning();
            }
        } catch (Exception e) {
            e.getMessage();
        }  finally {
            redisUtil.releaseLock(key,requestId);
        }
    }

    /**
     * 每天处理陌生人预警、常驻人员预警、昼伏夜出预警， 凌晨0点10分开始处理昨天数据
     */
    @Async
    @Scheduled(cron = "0 10 0 * * ?")
    public void handleDayWarning() {
        String key = RedisKeyConstant.HANDLE_DAY_WARNING;
        String requestId = UUID.randomUUID().toString();
        boolean result = redisUtil.tryLock(key,requestId,5 * 60);
        try {
            if (result) {
                warningService.handleDayWarning();
            }
        } catch (Exception e) {
            e.getMessage();
        }  finally {
            redisUtil.releaseLock(key,requestId);
        }
    }

}
