package com.iden.bms.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.iden.common.entity.IdenCommunity;

import com.iden.common.exceltool.CommonExcelParse;
import com.iden.common.exceltool.IExcelParser;
import com.iden.common.exceltool.MultipartFileToFile;
import com.iden.common.service.IdenCommunityService;
import com.iden.common.exceltool.ExcelUtil;
import com.iden.common.util.MyBeanUtils;
import com.iden.common.vo.CommunityVO;
import com.iden.common.vo.PageReqVO;
import com.iden.common.vo.UserLoginedConvertVO;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.*;

/**
 *
 * @author makejava
 * @since 2021-05-21 00:08:38
 */
@Service
public class CommunityService {

    @Resource
    private IdenCommunityService idenCommunityService;


    public Integer countCommunity(String district, String subdistrict, String name, UserLoginedConvertVO loginUser) {
        QueryWrapper<IdenCommunity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(StrUtil.isNotEmpty(name),IdenCommunity::getName,name)
                .eq(StrUtil.isNotEmpty(district),IdenCommunity::getDistrict,district)
                .eq(StrUtil.isNotEmpty(subdistrict),IdenCommunity::getSubdistrict,subdistrict);
        return this.idenCommunityService.count(queryWrapper);

    }

    /**
     * 查询小区列表
     * @return
             */
    public IPage<CommunityVO> listCommunity(String district, String subdistrict, String name, UserLoginedConvertVO loginUser, PageReqVO pageReqVo) {
            IPage<IdenCommunity> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
            QueryWrapper<IdenCommunity> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().like(StrUtil.isNotEmpty(name),IdenCommunity::getName,name)
                    .eq(StrUtil.isNotEmpty(district),IdenCommunity::getDistrict,district)
                    .eq(StrUtil.isNotEmpty(subdistrict),IdenCommunity::getSubdistrict,subdistrict)
                    .orderByAsc(IdenCommunity::getCode);

            IPage<IdenCommunity> pageRes = this.idenCommunityService.page(page, queryWrapper);
            IPage<CommunityVO> results = new Page<>(pageRes.getCurrent(),pageRes.getSize(),pageRes.getTotal());
            if(CollUtil.isNotEmpty(pageRes.getRecords())){
                List<CommunityVO> list = new ArrayList<>();
                pageRes.getRecords().forEach(item -> {
                    CommunityVO resVO = new CommunityVO();
                    BeanUtils.copyProperties(item,resVO);
                    list.add(resVO);
                });
                results.setRecords(list);
            }
            return results;
    }

    public void exportToExcel(String district, String subdistrict, String name, UserLoginedConvertVO loginUser, HttpServletResponse response)  throws Exception {
        QueryWrapper<IdenCommunity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(StrUtil.isNotEmpty(name),IdenCommunity::getName,name)
                .eq(StrUtil.isNotEmpty(district),IdenCommunity::getDistrict,district)
                .eq(StrUtil.isNotEmpty(subdistrict),IdenCommunity::getSubdistrict,subdistrict)
                .orderByAsc(IdenCommunity::getCode);
        List<IdenCommunity>  list = this.idenCommunityService.list(queryWrapper);
        List<CommunityVO> records = new ArrayList<>();
        if (CollUtil.isNotEmpty(list)) {
            list.forEach(item->{
                CommunityVO vo = new CommunityVO();
                BeanUtil.copyProperties(item, vo);
                records.add(vo);
            });
        }

        InputStream excelTemplateIs = null;
        try {
            response.reset(); // 非常重要
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");
            final String fileName = URLEncoder.encode("Community", "UTF-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + "_" + System.currentTimeMillis() + ".xlsx");

            // 模板注意 用{} 来表示你要用的变量 如果本来就有"{","}" 特殊字符 用"\{","\}"代替
            // {} 代表普通变量 {.} 代表是list的变量

            excelTemplateIs = this.getClass().getResourceAsStream("/static/template/export/IdenCommunityExportTemplate.xlsx");

            ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(excelTemplateIs).build();
            WriteSheet writeSheet = EasyExcel.writerSheet("小区表").build();
            // 这里注意 入参用了forceNewRow 代表在写入list的时候不管list下面有没有空行 都会创建一行，然后下面的数据往后移动。默认 是false，会直接使用下一行，如果没有则创建。
            // forceNewRow 如果设置了true,有个缺点 就是他会把所有的数据都放到内存了，所以慎用
            // 简单的说 如果你的模板有list,且list不是最后一行，下面还有数据需要填充 就必须设置 forceNewRow=true 但是这个就会把所有数据放到内存 会很耗内存
            // 如果数据量大 list不是最后一行 参照另一个
            FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.FALSE).build();
            excelWriter.fill(records, fillConfig, writeSheet);

            excelWriter.finish();

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if(excelTemplateIs != null){
                try {
                    excelTemplateIs.close();
                } catch (IOException ioe) {
                    ioe.printStackTrace();
                }
            }
        }
    }

    /**
     * 导入
     * @param multipartFile
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean importWithExcel(MultipartFile multipartFile) throws Exception {
        final File excelFile = MultipartFileToFile.multipartFileToFile(multipartFile);
        final Workbook workbook = ExcelUtil.chooseWorkbook(excelFile);
        final Sheet sheet = workbook.getSheetAt(0);
        final IExcelParser excelParser = new CommonExcelParse();
        final List<CommunityVO> listParser = excelParser.parse(sheet, new CommunityVO());
        if (CollectionUtil.isNotEmpty(listParser)) {
            List<IdenCommunity> listDb = new ArrayList<>();
            MultipartFileToFile.delteTempFile(excelFile);
            for(CommunityVO tmpObj : listParser){
                if(StringUtils.isEmpty(tmpObj.getCode())
                        || StringUtils.isEmpty(tmpObj.getName())
                        || StringUtils.isEmpty(tmpObj.getDistrict())
                        || StringUtils.isEmpty(tmpObj.getSubdistrict())){
                    continue;
                }
                final IdenCommunity obj =  new IdenCommunity();
                BeanUtils.copyProperties(tmpObj,obj);
                obj.setCreateTime(new Date());
                listDb.add(obj);
            }
            return this.idenCommunityService.saveBatch(listDb);
        }

        return false;
    }


    /**
     * 删除小区
     * @param id
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteById(Long id){
        return this.idenCommunityService.removeById(id);
    }

    /**
     * 详情
     * @param id
     * @return
     */
    public CommunityVO getCommunityById(Long id){
        IdenCommunity idenCommunity = this.idenCommunityService.getById(id);
        if (idenCommunity!=null){
            CommunityVO resVO = new CommunityVO();
            BeanUtil.copyProperties(idenCommunity,resVO);
            return resVO;
        }
        return null;
    }


    /**
     * 保存小区
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public int createCommunity(CommunityVO vo, UserLoginedConvertVO loginUser){
        QueryWrapper<IdenCommunity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(IdenCommunity::getName,vo.getName());
        if(idenCommunityService.count(queryWrapper) > 0){
            return 1;
        }
        QueryWrapper<IdenCommunity> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.lambda().eq(IdenCommunity::getCode,vo.getCode());
        if(idenCommunityService.count(queryWrapper2) > 0){
            return 2;
        }
        //保存小区
        IdenCommunity idenCommunity = new IdenCommunity();
        BeanUtil.copyProperties(vo,idenCommunity);
        idenCommunity.setCreateTime(new Date());

        this.idenCommunityService.save(idenCommunity);
        return 0;
    }

    /**
     * 修改小区
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public void updateCommunity(CommunityVO vo){
        //保存小区
        IdenCommunity idenCommunity = new IdenCommunity();
        BeanUtil.copyProperties(vo,idenCommunity);
        idenCommunity.setModifyTime(new Date());
        this.idenCommunityService.updateById(idenCommunity);

    }

}