package com.iden.bms.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.iden.common.entity.IdenCrowd;
import com.iden.common.entity.IdenFaceImage;
import com.iden.common.entity.IdenPerson;
import com.iden.common.entity.IdenPersonCrowdRef;
import com.iden.common.enums.PersonTypeEnum;
import com.iden.common.enums.PopulationTypeEnum;
import com.iden.common.service.IdenCrowdService;
import com.iden.common.service.IdenFaceImageService;
import com.iden.common.service.IdenPersonCrowdRefService;
import com.iden.common.service.IdenPersonService;

import com.iden.common.vo.FaceImageVO;
import com.iden.common.vo.PageReqVO;
import com.iden.common.vo.PersonVO;
import com.iden.common.vo.UserLoginedConvertVO;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author makejava
 * @since 2021-05-21 00:08:38
 */
@Service
public class FaceImageService {

    @Resource
    private IdenFaceImageService idenFaceImageService;
    @Resource
    private IdenPersonService idenPersonService;
    @Resource
    private IdenPersonCrowdRefService idenPersonCrowdRefService;
    @Resource
    private IdenCrowdService idenCrowdService;

    /**
     * 查询人脸抓拍库列表
     * @return
             */
    public IPage<FaceImageVO> listFaceImage(String type, Long communityId, String photographPlace, String beginDate, String endDate, UserLoginedConvertVO loginUser, PageReqVO pageReqVo) {
            IPage<IdenFaceImage> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
            QueryWrapper<IdenFaceImage> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().like(IdenFaceImage::getType,type)
                    .like(StrUtil.isNotEmpty(photographPlace),IdenFaceImage::getPhotographPlace,photographPlace)
                    .eq(communityId != null,IdenFaceImage::getCommunityId,communityId)
                    .orderByAsc(IdenFaceImage::getId);
            if(StrUtil.isNotEmpty(beginDate)){
                queryWrapper.apply(" data_date >= {0}", beginDate);
            }

            if(StrUtil.isNotEmpty(endDate)){
                queryWrapper.apply(" data_date <= {0}", endDate);
            }

            IPage<IdenFaceImage> pageRes = this.idenFaceImageService.page(page, queryWrapper);
            IPage<FaceImageVO> results = new Page<>(pageRes.getCurrent(),pageRes.getSize(),pageRes.getTotal());
            if(CollUtil.isNotEmpty(pageRes.getRecords())){
                List<FaceImageVO> list = new ArrayList<>();
                pageRes.getRecords().forEach(item -> {
                    FaceImageVO resVO = new FaceImageVO();
                    BeanUtils.copyProperties(item,resVO);
                    resVO.setTypeName(PersonTypeEnum.getValueToName(resVO.getType()));

                    list.add(resVO);
                });
                results.setRecords(list);
            }
            return results;
    }

    /**
     * 详情
     * @param id
     * @return
     */
    public FaceImageVO getFaceImageInfo(Long id){
        IdenFaceImage idenFaceImage = this.idenFaceImageService.getById(id);
        if (idenFaceImage != null){
            FaceImageVO resVO = new FaceImageVO();
            BeanUtil.copyProperties(idenFaceImage,resVO);

            if(resVO.getPersonId() != null){
                IdenPerson idenPerson = idenPersonService.getById(resVO.getPersonId());
                if(idenPerson != null) {
                    PersonVO personVO = new PersonVO();
                    BeanUtil.copyProperties(idenPerson,personVO);
                    personVO.setPopulationTypeName(PopulationTypeEnum.getValueToName(personVO.getPopulationType()));
                    personVO.setTypeName(PersonTypeEnum.getValueToName(personVO.getType()));

                    QueryWrapper<IdenPersonCrowdRef> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.lambda().eq(IdenPersonCrowdRef::getPersonId,personVO.getId());
                    List<IdenPersonCrowdRef> listIdenPersonCrowdRef = idenPersonCrowdRefService.list(queryWrapper1);
                    StringBuilder sb = new StringBuilder();
                    if (CollUtil.isNotEmpty(listIdenPersonCrowdRef)) {
                        for(IdenPersonCrowdRef idenPersonCrowdRef : listIdenPersonCrowdRef){
                            IdenCrowd idenCrowd = idenCrowdService.getById(idenPersonCrowdRef.getCrowdId());
                            if(idenCrowd != null) {
                                sb.append(idenCrowd.getName()).append(",");
                            }
                        }
                    }
                    String crowdName = sb.toString();
                    if(crowdName != null && crowdName.endsWith(",")) {
                        crowdName = crowdName.substring(0, crowdName.length() - 1);
                    }
                    personVO.setCrowdName(crowdName);
                    resVO.setPersonVO(personVO);
                }
            }

            return resVO;
        }
        return null;
    }

    /**
     * 根据图像文件名称得到IdenFaceImage
     * @param imageFileName
     * @return
     */
    public IdenFaceImage getIdenFaceImageByImageFileName(String imageFileName){

        QueryWrapper<IdenFaceImage> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(IdenFaceImage::getImage,imageFileName);
        IdenFaceImage idenFaceImage = idenFaceImageService.getOne(queryWrapper);
        return idenFaceImage;
    }

}