package com.iden.bms.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.iden.common.entity.*;
import com.iden.common.enums.*;
import com.iden.common.exception.BDException;
import com.iden.common.service.*;
import com.iden.common.util.MyBeanUtils;
import com.iden.common.vo.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 *
 * @author makejava
 * @since 2021-05-21 00:08:38
 */
@Service
public class WarningPersonService {

    @Resource
    private IdenPersonService idenPersonService;
    @Resource
    private IdenWarningPersonService idenWarningPersonService;
    @Resource
    private IdenWarningPersonHisService idenWarningPersonHisService;
    @Resource
    private IdenWarningEventService idenWarningEventService;
    @Resource
    private IdenPersonCrowdRefService idenPersonCrowdRefService;
    @Resource
    private IdenCrowdService idenCrowdService;
    @Resource
    private IdenCommunityService idenCommunityService;
    @Resource
    private IdenWarningPersonTraceService idenWarningPersonTraceService;
    @Resource
    private IdenCameraService idenCameraService;
    /**
     * 根据小区查询预警人员列表分页
     * @return
     */
    public IPage<WarningPersonVO> listWarningPersonbyCommunity(String type, Long communityId, String nameOrCred, String populationType, Long crowdId, String beginDate, String endDate,String handleStatus, UserLoginedConvertVO loginUser, PageReqVO pageReqVo) {
        QueryWrapper<IdenPerson> queryWrapper0 = new QueryWrapper<>();
        queryWrapper0.lambda().eq(StrUtil.isNotEmpty(populationType),IdenPerson::getPopulationType,populationType)
                .and(StrUtil.isNotEmpty(nameOrCred),wrapper -> wrapper.like(IdenPerson::getName,nameOrCred)
                .or().like(IdenPerson::getCredentialsCode,nameOrCred));
        if (crowdId != null) {
            queryWrapper0.apply(" EXISTS ( select dev_code from iden_person_crowd_ref ipcr where ipcr.person_id = iden_person.id  and ipcr.crowd_id  = " + crowdId + ")");
        }

        List<IdenPerson> listIdenPerson = idenPersonService.list(queryWrapper0);
        List<Long> listIdenPersonId = new ArrayList<>();
        if(CollUtil.isNotEmpty(listIdenPerson)){
            listIdenPerson.forEach(item->{
                listIdenPersonId.add(item.getId());
            });
        }

        IPage<IdenWarningPerson> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
        QueryWrapper<IdenWarningPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(IdenWarningPerson::getType,type).eq(IdenWarningPerson::getCommunityId,communityId)
                .ge(StrUtil.isNotEmpty(beginDate),IdenWarningPerson::getDataDate,beginDate)
                .le(StrUtil.isNotEmpty(endDate),IdenWarningPerson::getDataDate,endDate)
                .eq(StrUtil.isNotEmpty(handleStatus),IdenWarningPerson::getHandleStatus,handleStatus)
                .in(CollUtil.isNotEmpty(listIdenPersonId),IdenWarningPerson::getPersonId,listIdenPersonId)
                .orderByAsc(IdenWarningPerson::getId);

        IPage<IdenWarningPerson> pageRes = this.idenWarningPersonService.page(page, queryWrapper);
        IPage<WarningPersonVO> results = new Page<>(pageRes.getCurrent(),pageRes.getSize(),pageRes.getTotal());
        if(CollUtil.isNotEmpty(pageRes.getRecords())){
            List<WarningPersonVO> list = new ArrayList<>();
            pageRes.getRecords().forEach(item -> {
                WarningPersonVO resVO = new WarningPersonVO();
                BeanUtils.copyProperties(item,resVO);
                resVO.setTypeName(WarningTypeEnum.getValueToName(resVO.getType()));
                resVO.setStrangerTypeName(StrangerTypeEnum.getValueToName(resVO.getStrangerType()));
                resVO.setIsContinueName(IsContinueEnum.getValueToName(resVO.getIsContinue()));
                resVO.setHandleStatusName(HandleWarningStatusEnum.getValueToName(resVO.getHandleStatus()));

                if(resVO.getCrowdId() != null){
                    IdenCrowd idenCrowd = idenCrowdService.getById(resVO.getCrowdId());
                    if(idenCrowd != null){
                        resVO.setCrowdName(idenCrowd.getName());
                    }
                }

                Long personId = resVO.getPersonId();
                if (personId != null) {
                    IdenPerson idenPerson = this.idenPersonService.getById(personId);
                    PersonVO personVO = new PersonVO();
                    BeanUtils.copyProperties(idenPerson,personVO);
                    personVO.setPopulationTypeName(PopulationTypeEnum.getValueToName(personVO.getPopulationType()));
                    personVO.setTypeName(PersonTypeEnum.getValueToName(personVO.getType()));

                    QueryWrapper<IdenPersonCrowdRef> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.lambda().eq(IdenPersonCrowdRef::getPersonId,personVO.getId());
                    List<IdenPersonCrowdRef> listIdenPersonCrowdRef = idenPersonCrowdRefService.list(queryWrapper1);
                    StringBuilder sb = new StringBuilder();
                    if (CollUtil.isNotEmpty(listIdenPersonCrowdRef)) {
                        for(IdenPersonCrowdRef idenPersonCrowdRef : listIdenPersonCrowdRef){
                            IdenCrowd idenCrowd = idenCrowdService.getById(idenPersonCrowdRef.getCrowdId());
                            if(idenCrowd != null) {
                                sb.append(idenCrowd.getName()).append(",");
                            }
                        }
                    }
                    String crowdName = sb.toString();
                    if(crowdName != null && crowdName.endsWith(",")) {
                        crowdName = crowdName.substring(0, crowdName.length() - 1);
                    }
                    personVO.setCrowdName(crowdName);
                    resVO.setPersonVO(personVO);
                }
                list.add(resVO);
            });
            results.setRecords(list);
        }
        return results;
    }

    /**
     *  查询轨迹列表
     * @return
     */
    public List<TraceVO> listTrace(String uid, String beginDate, String endDate, UserLoginedConvertVO loginUser) {
        List<TraceVO> vos = new ArrayList<>();
        QueryWrapper<IdenWarningPersonTrace> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(IdenWarningPersonTrace::getUid,uid)
                .ge(StringUtils.isNotEmpty(beginDate),IdenWarningPersonTrace::getDataDate,beginDate)
                .le(StringUtils.isNotEmpty(endDate),IdenWarningPersonTrace::getDataDate,endDate)
                .orderByAsc(IdenWarningPersonTrace::getPhotographTime);
        List<IdenWarningPersonTrace> idenWarningPersonTraceList = this.idenWarningPersonTraceService.list(queryWrapper);
        if(CollUtil.isNotEmpty(idenWarningPersonTraceList)) {
            idenWarningPersonTraceList.forEach(item->{
                TraceVO vo = new TraceVO();
                IdenWarningPersonTrace idenWarningPersonTrace = new IdenWarningPersonTrace();
                BeanUtil.copyProperties(idenWarningPersonTrace,vo);
                if(vo.getCameraId() != null){
                    IdenCamera idenCamera = this.idenCameraService.getById(vo.getCameraId());
                    if(idenCamera != null) {
                        vo.setDistrict(idenCamera.getDistrict());
                        vo.setSubdistrict(idenCamera.getSubdistrict());

                        Long communityId = idenCamera.getCommunityId();
                        if(communityId != null ){
                            IdenCommunity idenCommunity = this.idenCommunityService.getById(communityId);
                            if(idenCommunity != null) {
                                vo.setCommunityName(idenCommunity.getName());
                            }
                        }
                    }
                }
                vos.add(vo);
            });
        }
        return vos;
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean handleWarning(WarningPersonVO vo) throws BDException {

        if(vo.getId() == null || StringUtils.isEmpty(vo.getType())){
            return false;
        }

        IdenWarningPerson idenWarningPerson = this.idenWarningPersonService.getById(vo.getId());

        if(WarningTypeEnum.STRANGER.getValue().equals(vo.getType())) {
            if (1 == vo.getObtainPersonInfo()){ //已经获得本人信息
                //生成人员数据
                IdenPerson idenPerson = new IdenPerson();
                idenPerson.setName(vo.getPersonName());
                idenPerson.setUid(idenWarningPerson.getUid());
                idenPerson.setImage(idenWarningPerson.getImage());
                idenPerson.setFeatPtr(idenWarningPerson.getFeatPtr());
                idenPerson.setCommunityId(idenWarningPerson.getCommunityId());
                idenPerson.setDistrict(idenWarningPerson.getDistrict());
                idenPerson.setSubdistrict(idenWarningPerson.getSubdistrict());
                idenPerson.setCreateTime(new Date());
                idenPerson.setType(PersonTypeEnum.COMMUNITY.getValue());
                this.idenPersonService.save(idenPerson);
            }
        }
        MyBeanUtils.copyProperties(vo,idenWarningPerson);
        idenWarningPerson.setModifyTime(new Date());
        idenWarningPerson.setHandleStatus(HandleWarningStatusEnum.HANDLED.getValue());

        //生成一套历史数据
        IdenWarningPersonHis idenWarningPersonHis = new IdenWarningPersonHis();
        BeanUtil.copyProperties(idenWarningPerson,idenWarningPersonHis,"id");
        idenWarningPersonHisService.save(idenWarningPersonHis);

        return idenWarningPersonService.updateById(idenWarningPerson);


    }

    public List<WarningEventVO> listEventTimes(Long id, UserLoginedConvertVO loginUser){
        List<WarningEventVO> res = new ArrayList<>();
        QueryWrapper<IdenWarningEvent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(IdenWarningEvent::getWarningPersonId,id).orderByAsc(IdenWarningEvent::getAppearTime);
        List<IdenWarningEvent> list = this.idenWarningEventService.list(queryWrapper);
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                WarningEventVO vo = new WarningEventVO();
                BeanUtil.copyProperties(item,vo);
                res.add(vo);
            });
        }
        return res;
    }
}