package com.iden.common.config;

import com.zaxxer.hikari.HikariDataSource;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.sql.DataSource;


/**
 * @author javadoc
 * @version 1.0
 * @description 数据库DataSource配置 废弃springboot自动装配,采用自定义配置,旨在为了去掉配置文件中password关键字
 * @date 2021/02/08/19:41
 */
@Configuration
public class DataSourceConfig {
    @Value("${spring.datasource.url}")
    private String url;
    @Value("${spring.datasource.auth}")
    private String auth;
    /**
     *以下是hikari的配置 如果项目组有额外的其他配置请一并增加进来并在下面进行设置 否则不起作用
     */
    @Value("${spring.datasource.hikari.maximum-pool-size}")
    private int maximumPoolSize;
    @Value("${spring.datasource.hikari.minimum-idle}")
    private int minimumIdle;
    @Value("${spring.datasource.hikari.connection-timeout}")
    private long connectionTimeout;
    @Value("${spring.datasource.hikari.connection-test-query}")
    private String testQuery;

    @Bean
    @ConfigurationProperties("spring.datasource")
    public DataSource getDataSource() {
        HikariDataSource dataSource =
                DataSourceBuilder.create().type(HikariDataSource.class).build();
        dataSource.setJdbcUrl(url);
        dataSource.setPassword(auth);
        dataSource.setPoolName("HikariDataSourcePool");
        dataSource.setMaximumPoolSize(maximumPoolSize);
        dataSource.setMinimumIdle(minimumIdle);
        dataSource.setConnectionTimeout(connectionTimeout);
        dataSource.setConnectionTestQuery(testQuery);
        return dataSource;
    }
}
