package com.iden.common.interceptor;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.iden.common.constant.Constants;
import com.iden.common.vo.UserLoginedConvertVO;
import com.iden.common.util.JwtUtils;
import io.jsonwebtoken.Claims;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 描述:
 *
 * @author y
 * @create 2018-10-18 15:11
 */
@Component
public class AuthorizationInterceptor extends HandlerInterceptorAdapter {


    /**
     * 解析请求
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        /**
         * 获取登录用户
         */
        Claims claims = parseToken(request);
        if (claims != null) {
            UserLoginedConvertVO dto = JSONUtil.toBean(claims.getSubject(), UserLoginedConvertVO.class);
            if (dto != null) {
                /**
                 * 设置登录用户到请求 Request 中.
                 */
                request.setAttribute(Constants.CURRENT_LOGINED_USER_KEY, dto);
            }
        }
        return super.preHandle(request, response, handler);
    }

    /**
     * 获取登录用户
     * @param request
     * @return
     */
    private Claims parseToken(HttpServletRequest request) {
        /**
         * 获取Header中携带TOKEN
         */
        String commonJwt = request.getHeader(Constants.LOGIN_TOKEN_KEY);
        if (StrUtil.isBlank(commonJwt)) {
            /**
             * 获取参数中携带TOKEN
             */
            commonJwt = request.getParameter(Constants.LOGIN_TOKEN_KEY);
            if (StrUtil.isBlank(commonJwt)) {
                return null;
            }
        }
        Claims claims = JwtUtils.tokenParse(commonJwt);
        if(claims != null){
            return claims;
        }
        return null;
    }

}
